'''
 For testing iDynTree python bindings, we rely on the unittest standard python lib
'''

import sys

# This test is meant to be executed from the build,
# so we add in sys.path the location of iDynTree.py and _iDynTree.so
# TODO: this probably does not work on Windows, fix it
sys.path.append("../../python/")
sys.path.append("../../../lib/python/")
sys.path.append("../../../lib/python/Debug/")

import unittest
import idyntree.swig as iDynTree
import random

class JointTest(unittest.TestCase):
    '''tests'''
    def testPrismaticJoint(self):
        prismJoint = iDynTree.PrismaticJoint();

    def testRevoluteJoint(self):
        revJoint = iDynTree.RevoluteJoint();

    def testFixedJoint(self):
        fixJoint = iDynTree.FixedJoint(iDynTree.Transform.Identity());

if __name__ == '__main__':
    # initalize the seed to have predictable results
    random.seed(0);
    unittest.main()



"""Tests for idyntree-model-io-urdf Python bindings."""
import os
import tempfile
import unittest

import idyntree.pybind as iDynTree


class IDynTreeModelIoUrdfTest(unittest.TestCase):

  def test_exporter_options(self):
    options = iDynTree.ModelExporterOptions()
    options.base_link = "a_base_link"
    options.robot_exported_name = "robot_name"
    options.export_first_base_link_additional_frame_as_fake_urdfbase = True
    self.assertEqual(options.base_link, "a_base_link")
    self.assertEqual(options.robot_exported_name, "robot_name")
    self.assertTrue(
        options.export_first_base_link_additional_frame_as_fake_urdfbase)
    options.export_first_base_link_additional_frame_as_fake_urdfbase = False
    self.assertFalse(
        options.export_first_base_link_additional_frame_as_fake_urdfbase)

  def test_exporter(self):
    exporter = iDynTree.ModelExporter()
    model = iDynTree.Model()
    link = iDynTree.Link()
    model.add_link("a_link", link)
    exporter.init(model)
    self.assertTrue(exporter.export_model_to_string())
    self.assertEqual(exporter.model.get_nr_of_links(), model.get_nr_of_links())

  def test_custom_options(self):
    options = iDynTree.ModelExporterOptions()
    options.robot_exported_name = "robot_name"
    exporter = iDynTree.ModelExporter()
    model = iDynTree.Model()
    link = iDynTree.Link()
    model.add_link("a_link", link)
    exporter.init(model, options=options)
    self.assertEqual(exporter.options.robot_exported_name, "robot_name")

  def test_custom_sensors(self):
    exporter = iDynTree.ModelExporter()
    model = iDynTree.Model()
    link = iDynTree.Link()
    model.add_link("a_link", link)
    sensors = iDynTree.SensorsList()
    sensor = iDynTree.AccelerometerSensor()
    sensor.name = "a_name"
    sensor.parent_link_index = 1
    sensors.add_sensor(sensor)
    exporter.init(model, sensors=sensors)
    self.assertEqual(
        exporter.sensors.get_nr_of_sensors(iDynTree.ACCELEROMETER), 1)

  def test_importer(self):
    model = (r'<robot name="test_robot">'
             r'<link name="link1" />'
             r'<link name="link2" />'
             r'<link name="link3" />'
             r'<link name="link4" />'
             r'<joint name="joint1" type="continuous">'
             r'<parent link="link1"/>'
             r'<child link="link2"/>'
             r'</joint>'
             r'<joint name="joint2" type="continuous">'
             r'<parent link="link1"/>'
             r'<child link="link3"/>'
             r'</joint>'
             r'<joint name="joint3" type="continuous">'
             r'<parent link="link3"/>'
             r'<child link="link4"/>'
             r'</joint>'
             r'</robot>')

    tmpdir = tempfile.TemporaryDirectory()
    urdf_filename = os.path.join(tmpdir.name, 'model.urdf')
    with open(urdf_filename, 'w') as f:
      f.write(model)

    importer = iDynTree.ModelLoader()
    importer.load_model_from_file(urdf_filename)
    self.assertEqual(4, importer.model.get_nr_of_links())
    self.assertEqual(3, importer.model.get_nr_of_joints())

  def test_importer_reduced_model(self):
    model = (r'<robot name="test_robot">'
             r'<link name="link1" />'
             r'<link name="link2" />'
             r'<link name="link3" />'
             r'<link name="link4" />'
             r'<joint name="joint1" type="continuous">'
             r'<parent link="link1"/>'
             r'<child link="link2"/>'
             r'</joint>'
             r'<joint name="joint2" type="continuous">'
             r'<parent link="link1"/>'
             r'<child link="link3"/>'
             r'</joint>'
             r'<joint name="joint3" type="continuous">'
             r'<parent link="link3"/>'
             r'<child link="link4"/>'
             r'</joint>'
             r'</robot>')

    tmpdir = tempfile.TemporaryDirectory()
    urdf_filename = os.path.join(tmpdir.name, 'model.urdf')
    with open(urdf_filename, 'w') as f:
      f.write(model)

    importer = iDynTree.ModelLoader()
    importer.load_model_from_file(urdf_filename)
    importer.load_reduced_model_from_full_model(importer.model,
                                                ('joint1', 'joint3'))
    # The reduced model will have the 2 joints we specified, and one less body
    # as the missing joint made two bodies being merged together.
    self.assertEqual(3, importer.model.get_nr_of_links())
    self.assertEqual(2, importer.model.get_nr_of_joints())
    self.assertEqual('joint1', importer.model.get_joint_name(0))
    self.assertEqual('joint3', importer.model.get_joint_name(1))


if __name__ == "__main__":
  unittest.main()



#!/usr/bin/env python3


"""
Copyright (C) 2022 Fondazione Istituto Italiano di Tecnologia

Licensed under either the GNU Lesser General Public License v3.0 :
https://www.gnu.org/licenses/lgpl-3.0.html
or the GNU Lesser General Public License v2.1 :
https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
at your option.
"""

import argparse

from idyntree.visualize import MeshcatVisualizer


def main():
    parser = argparse.ArgumentParser(description='Display the model in the meshcat visualizer.')
    parser.add_argument('--model', '-m', type = str, required = True, help='Model path.')

    args = parser.parse_args()

    # Load the visualizer
    viz = MeshcatVisualizer()
    viz.set_model_from_file(args.model)
    viz.load_model()
    viz.open()

    run = True
    while run:
        quit = input('Enter q or Q to Quit: ')
        run = (quit != 'q') and (quit != 'Q')


if __name__ == "__main__":
    main()



"""
Copyright (C) 2021 Fondazione Istituto Italiano di Tecnologia

Licensed under either the GNU Lesser General Public License v3.0 :
https://www.gnu.org/licenses/lgpl-3.0.html
or the GNU Lesser General Public License v2.1 :
https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
at your option.
"""

import os
import idyntree.bindings as idyn
import numpy as np
import warnings
from pathlib import Path


class MeshcatVisualizer:
    """
    A simple wrapper to the meshcat visualizer. The MeshcatVisualizer class is highly inspired by the Pinocchio version
    of the MeshCat visualizer
    https://github.com/stack-of-tasks/pinocchio/blob/b134b25f1409f5bf036105b996da2d29c1a66a12/bindings/python/pinocchio/visualize/meshcat_visualizer.py
    """

    def __init__(self, zmq_url=None):
        import meshcat

        if zmq_url is not None:
            print("Connecting to meshcat-server at zmq_url=" + zmq_url + ".")

        self.viewer = meshcat.Visualizer(zmq_url=zmq_url)
        self.traversal = dict()
        self.model = dict()
        self.link_pos = dict()
        self.primitive_geometries_names = []

    @staticmethod
    def __is_mesh(geometry_object: idyn.SolidShape) -> bool:

        if not geometry_object.isExternalMesh():
            return False
        mesh_path = geometry_object.asExternalMesh().getFileLocationOnLocalFileSystem()

        # Check whether the geometry object contains a Mesh supported by MeshCat
        if mesh_path == "":
            return False

        _, file_extension = os.path.splitext(mesh_path)
        if file_extension.lower() in [".dae", ".obj", ".stl"]:
            return True

        return False

    @staticmethod
    def __load_mesh(geometry_object: idyn.SolidShape):
        import meshcat

        mesh_path = geometry_object.asExternalMesh().getFileLocationOnLocalFileSystem()

        # try to import the mesh
        if mesh_path == "":
            return None

        _, file_extension = os.path.splitext(mesh_path)

        basename = os.path.basename(mesh_path)
        file_name = os.path.splitext(basename)[0]

        geometry_object.asExternalMesh().setName(file_name)

        if file_extension.lower() == ".dae":
            obj = meshcat.geometry.DaeMeshGeometry.from_file(mesh_path)
        elif file_extension.lower() == ".obj":
            obj = meshcat.geometry.ObjMeshGeometry.from_file(mesh_path)
        elif file_extension.lower() == ".stl":
            obj = meshcat.geometry.StlMeshGeometry.from_file(mesh_path)
        else:
            msg = "The following mesh cannot be loaded: {}.".format(mesh_path)
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            obj = None

        return obj

    @staticmethod
    def __load_primitive_geometry(geometry_object: idyn.SolidShape):
        import meshcat

        # Cylinders need to be rotated
        R = np.array(
            [
                [1.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, -1.0, 0.0],
                [0.0, 1.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 1.0],
            ]
        )
        RotatedCylinder = type(
            "RotatedCylinder",
            (meshcat.geometry.Cylinder,),
            {"intrinsic_transform": lambda self: R},
        )

        if geometry_object.isCylinder():
            obj = RotatedCylinder(
                geometry_object.asCylinder().getLength(),
                geometry_object.asCylinder().getRadius(),
            )
        elif geometry_object.isBox():
            obj = meshcat.geometry.Box(
                (
                    geometry_object.asBox().getX(),
                    geometry_object.asBox().getY(),
                    geometry_object.asBox().getZ(),
                )
            )
        elif geometry_object.isSphere():
            obj = meshcat.geometry.Sphere(geometry_object.asSphere().getRadius())
        else:
            msg = "Unsupported geometry type (%s)" % type(geometry_object)
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            obj = None

        return obj

    def __apply_transform_to_primitive_geomety(
        self, world_H_frame, solid_shape, viewer_name
    ):
        world_H_geometry = (
            (world_H_frame * solid_shape.getLink_H_geometry())
            .asHomogeneousTransform()
            .toNumPy()
        )
        world_H_geometry_scaled = np.array(world_H_geometry)

        # Update viewer configuration.
        self.viewer[viewer_name].set_transform(world_H_geometry_scaled)

    def __apply_transform(self, world_H_frame, solid_shape, viewer_name):
        world_H_geometry = (
            (world_H_frame * solid_shape.getLink_H_geometry())
            .asHomogeneousTransform()
            .toNumPy()
        )
        scale = list(solid_shape.asExternalMesh().getScale().toNumPy().flatten())
        extended_scale = np.diag(np.concatenate((scale, [1.0])))
        world_H_geometry_scaled = np.array(world_H_geometry).dot(extended_scale)

        # Update viewer configuration.
        self.viewer[viewer_name].set_transform(world_H_geometry_scaled)

    def __model_exists(self, model_name):
        return (
            model_name in self.model.keys()
            or model_name in self.traversal.keys()
            or model_name in self.link_pos.keys()
        )

    def __primitive_geometry_exists(self, geometry_name: str):
        return geometry_name in self.primitive_geometries_names

    def __add_model_geometry_to_viewer(
        self, model_geometry: idyn.ModelSolidShapes, model_name: str, color: list
    ):
        import meshcat

        if not self.__model_exists(model_name):
            msg = "The model named: " + model_name + " does not exist."
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        # Solve forward kinematics
        joint_pos = idyn.VectorDynSize(self.model[model_name].getNrOfJoints())
        joint_pos.zero()
        idyn.ForwardPositionKinematics(
            self.model[model_name],
            self.traversal[model_name],
            idyn.Transform.Identity(),
            joint_pos,
            self.link_pos[model_name],
        )

        link_solid_shapes = model_geometry.getLinkSolidShapes()

        for link_index in range(0, self.model[model_name].getNrOfLinks()):

            world_H_frame = self.link_pos[model_name](link_index)
            link_name = self.model[model_name].getLinkName(link_index)

            for geom in range(0, len(link_solid_shapes[link_index])):
                solid_shape = model_geometry.getLinkSolidShapes()[link_index][geom]

                is_mesh = False
                is_primitive_geomety = False

                if self.__is_mesh(solid_shape):
                    obj = self.__load_mesh(solid_shape)
                    is_mesh = True
                else:
                    obj = self.__load_primitive_geometry(solid_shape)
                    is_primitive_geomety = True

                if obj is None:
                    msg = (
                        "The geometry object named "
                        + solid_shape.asExternalMesh().getName()
                        + " is not valid."
                    )
                    warnings.warn(msg, category=UserWarning, stacklevel=2)
                    continue

                if is_mesh:
                    viewer_name = (
                        model_name
                        + "/"
                        + link_name
                        + "/"
                        + solid_shape.asExternalMesh().getName()
                    )
                else:
                    viewer_name = model_name + "/" + link_name + "/geometry" + str(geom)

                if isinstance(obj, meshcat.geometry.Object):
                    self.viewer[viewer_name].set_object(obj)
                elif isinstance(obj, meshcat.geometry.Geometry):
                    material = meshcat.geometry.MeshPhongMaterial()
                    # Set material color from URDF, converting for triplet of doubles to a single int.
                    if color is None:
                        mesh_color = solid_shape.getMaterial().color()
                    else:
                        mesh_color = color

                    material.color = (
                        int(mesh_color[0] * 255) * 256 ** 2
                        + int(mesh_color[1] * 255) * 256
                        + int(mesh_color[2] * 255)
                    )

                    # Add transparency, if needed.
                    if float(mesh_color[3]) != 1.0:
                        material.transparent = True
                        material.opacity = float(mesh_color[3])

                    self.viewer[viewer_name].set_object(obj, material)

                    if is_mesh:
                        self.__apply_transform(world_H_frame, solid_shape, viewer_name)
                    elif is_primitive_geomety:
                        self.__apply_transform_to_primitive_geomety(
                            world_H_frame, solid_shape, viewer_name
                        )

    def set_multibody_system_state(
        self, base_position, base_rotation, joint_value, model_name="iDynTree"
    ):
        """Display the robot at given configuration."""

        if not self.__model_exists(model_name):
            msg = "The multi-body system named: " + model_name + " does not exist."
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        base_rotation_idyn = idyn.Rotation()
        base_position_idyn = idyn.Position()
        base_pose_idyn = idyn.Transform()

        for i in range(0, 3):
            base_position_idyn.setVal(i, base_position[i])
            for j in range(0, 3):
                base_rotation_idyn.setVal(i, j, base_rotation[i, j])

        base_pose_idyn.setRotation(base_rotation_idyn)
        base_pose_idyn.setPosition(base_position_idyn)

        if len(joint_value) != self.model[model_name].getNrOfJoints():
            msg = "The size of the joint_values is different from the model DoFs"
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        joint_pos_idyn = idyn.VectorDynSize(self.model[model_name].getNrOfJoints())
        for i in range(0, self.model[model_name].getNrOfJoints()):
            joint_pos_idyn.setVal(i, joint_value[i])

        # Solve forward kinematics
        idyn.ForwardPositionKinematics(
            self.model[model_name],
            self.traversal[model_name],
            base_pose_idyn,
            joint_pos_idyn,
            self.link_pos[model_name],
        )

        # Update the visual shapes
        model_geometry = self.model[model_name].visualSolidShapes()
        link_solid_shapes = model_geometry.getLinkSolidShapes()

        for link_index in range(0, self.model[model_name].getNrOfLinks()):

            link_name = self.model[model_name].getLinkName(link_index)
            for geom in range(0, len(link_solid_shapes[link_index])):
                solid_shape = model_geometry.getLinkSolidShapes()[link_index][geom]
                if self.__is_mesh(solid_shape):
                    viewer_name = (
                        model_name
                        + "/"
                        + link_name
                        + "/"
                        + solid_shape.asExternalMesh().getName()
                    )
                    self.__apply_transform(
                        self.link_pos[model_name](link_index), solid_shape, viewer_name
                    )
                # is a primitive shape
                else:
                    viewer_name = model_name + "/" + link_name + "/geometry" + str(geom)
                    self.__apply_transform_to_primitive_geomety(
                        self.link_pos[model_name](link_index), solid_shape, viewer_name
                    )

    def open(self):
        self.viewer.open()

    def jupyter_cell(self):
        return self.viewer.jupyter_cell()

    def load_primitive_geometry(self, solid_shape, shape_name="iDynTree", color=None):
        import meshcat

        # check if the model already exist
        if self.__primitive_geometry_exists(shape_name) or self.__model_exists(
            shape_name
        ):
            msg = "The model named: " + shape_name + " already exists."
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        # try to load the primitive
        obj = self.__load_primitive_geometry(solid_shape)

        if obj is None:
            msg = "The geometry object named " + shape_name + " is not valid."
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        viewer_name = shape_name

        if isinstance(obj, meshcat.geometry.Object):
            self.viewer[viewer_name].set_object(obj)
        elif isinstance(obj, meshcat.geometry.Geometry):
            material = meshcat.geometry.MeshPhongMaterial()
            # Set material color from URDF, converting for triplet of doubles to a single int.
            if color is None:
                mesh_color = solid_shape.getMaterial().color()
            else:
                mesh_color = color

            material.color = (
                int(mesh_color[0] * 255) * 256 ** 2
                + int(mesh_color[1] * 255) * 256
                + int(mesh_color[2] * 255)
            )

            # Add transparency, if needed.
            if float(mesh_color[3]) != 1.0:
                material.transparent = True
                material.opacity = float(mesh_color[3])

            self.viewer[viewer_name].set_object(obj, material)
            self.primitive_geometries_names.append(viewer_name)

    def set_primitive_geometry_transform(
        self, position, rotation, shape_name="iDynTree"
    ):
        if self.__primitive_geometry_exists(shape_name):
            transform = np.zeros((4, 4))
            transform[0:3, 0:3] = rotation
            transform[0:3, 3] = position
            transform[3, 3] = 1
            self.viewer[shape_name].set_transform(transform)

    def load_model_from_file(
        self, model_path: str, considered_joints=None, model_name="iDynTree", color=None
    ):

        p = Path(model_path)
        path_str = str(p.absolute())
        model_loader = idyn.ModelLoader()
        if considered_joints is None:
            ok = model_loader.loadModelFromFile(path_str)

        else:
            considered_joints_idyn = idyn.StringVector()
            for joint in considered_joints:
                considered_joints_idyn.push_back(joint)

            ok = model_loader.loadReducedModelFromFile(path_str, considered_joints_idyn)

        if not ok:
            msg = (
                "Unable to load the model named: "
                + model_name
                + " from the file: "
                + model_path
                + "."
            )
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        self.load_model(model=model_loader.model(), model_name=model_name, color=color)

    def load_model(self, model: idyn.Model, model_name="iDynTree", color=None):

        # check if the model already exist
        if self.__model_exists(model_name):
            msg = "The model named: " + model_name + " already exists."
            warnings.warn(msg, category=UserWarning, stacklevel=2)
            return

        self.model[model_name] = model.copy()
        self.traversal[model_name] = idyn.Traversal()
        self.link_pos[model_name] = idyn.LinkPositions()

        self.model[model_name].computeFullTreeTraversal(self.traversal[model_name])
        self.link_pos[model_name].resize(self.model[model_name])

        self.__add_model_geometry_to_viewer(
            model_geometry=self.model[model_name].visualSolidShapes(),
            model_name=model_name,
            color=color,
        )

    def load_sphere(self, radius, shape_name="iDynTree", color=None):
        sphere = idyn.Sphere()
        sphere.setRadius(radius)
        self.load_primitive_geometry(
            solid_shape=sphere, shape_name=shape_name, color=color
        )

    def load_cylinder(self, radius, length, shape_name="iDynTree", color=None):
        cylinder = idyn.Cylinder()
        cylinder.setRadius(radius)
        cylinder.setLength(length)
        self.load_primitive_geometry(
            solid_shape=cylinder, shape_name=shape_name, color=color
        )

    def load_box(self, x, y, z, shape_name="iDynTree", color=None):
        box = idyn.Box()
        box.setX(x)
        box.setY(y)
        box.setZ(z)
        self.load_primitive_geometry(
            solid_shape=box, shape_name=shape_name, color=color
        )



'''
 For testing iDynTree python bindings, we rely on the unittest standard python lib
'''

import sys

# This test is meant to be executed from the build,
# so we add in sys.path the location of iDynTree.py and _iDynTree.so
# TODO: this probably does not work on Windows, fix it
sys.path.append("../../python/")
sys.path.append("../../../lib/python/")
sys.path.append("../../../lib/python/Debug/")

import unittest
import idyntree.swig as iDynTree
import random

class PositionTest(unittest.TestCase):
    '''Helper methods'''
    def tol(self):
        return 1e-10;

    def places(self):
        return 7;

    def checkApproxEqual(self,val1,val2,msg):
        self.assertAlmostEqual(val1,val2,self.places(),msg);

    def checkPointEqual(self,p1,p2,msg):
        msgMore = "val1 = " + p1.toString() + " and val2 = " + p2.toString();
        for i in range(0,3):
            self.checkApproxEqual(p1.getVal(i),p2.getVal(i),msg+":"+msgMore);

    def checkSpatialVectorEqual(self,p1,p2,msg):
        msgMore = "val1 = " + p1.toString() + " and val2 = " + p2.toString();
        for i in range(0,3):
            self.checkApproxEqual(p1.getVal(i),p2.getVal(i),msg+":"+msgMore);

    def randomTransform(self):
        ''' Return a random Transform '''
        res = iDynTree.Transform();
        res.setPosition(iDynTree.Position(random.uniform(-10,10),random.uniform(-10,10),random.uniform(-10,10)));
        res.setRotation(iDynTree.Rotation.RPY(random.uniform(-10,10),random.uniform(-10,10),random.uniform(-10,10)));
        return res;

    def randomTwist(self):
        ''' Return a random Twist '''
        res = iDynTree.Twist()
        for i in range(0,6):
            res.setVal(i,random.uniform(-10,10));
        return res;

    def randomWrench(self):
        ''' Return a random wrench '''
        res = iDynTree.Wrench()
        for i in range(0,6):
            res.setVal(i,random.uniform(-10,10));
        return res;

    def randomPosition(self):
        '''Return a random position vector'''
        res = iDynTree.Position()
        for i in range(0,3):
            res.setVal(i,random.uniform(-10,10));

        return res

    '''tests'''

    def testInnerProductInvariance(self, nrOfTests=5):
        ''' Check that the inner product between a wrench and a twist
        is invariant with respect to adjoint transforms'''
        print("Running test testInnerProductInvariance");
        for i in range(0,nrOfTests):
            f = self.randomWrench();
            v = self.randomTwist();
            T = self.randomTransform();
            power1 = f.dot(v);
            power2 = v.dot(f);
            power3 = (T*f).dot(T*v);
            power4 = (T*v).dot(T*f);
            T_id = T*T.inverse();
            power5 = (T_id*f).dot(T_id*v);
            power6 = (T_id*v).dot(T_id*f);
            power7 = (T.inverse()*f).dot(T.inverse()*v);
            power8 = (T.inverse()*v).dot(T.inverse()*f)
            self.checkApproxEqual(power1,power2,"testInnerProductInvariance failed");
            self.checkApproxEqual(power1,power3,"testInnerProductInvariance failed");
            self.checkApproxEqual(power1,power4,"testInnerProductInvariance failed");
            self.checkApproxEqual(power1,power5,"testInnerProductInvariance failed");
            self.checkApproxEqual(power1,power6,"testInnerProductInvariance failed");
            self.checkApproxEqual(power1,power7,"testInnerProductInvariance failed");
            self.checkApproxEqual(power1,power8,"testInnerProductInvariance failed");


    def testTransformInverse(self,nrOfTests=5):
        print("Running test testTransformInverse");
        for i in range(0,nrOfTests):
            #RTF.testReport("Running test testTransformInverse for position");
            p = self.randomPosition()
            T = self.randomTransform()
            pZero = iDynTree.Position()
            pZero.zero()
            self.checkPointEqual(p-p,pZero,"testTransformInverse failed");
            self.checkPointEqual(T.inverse()*T*p,p,"testTransformInverse failed");
            #RTF.testReport("Running test testTransformInverse for Twist");
            v = self.randomTwist()
            vZero = iDynTree.Twist()
            vZero.zero()
            self.checkSpatialVectorEqual(v-v,vZero,"v-v is not zero");
            self.checkSpatialVectorEqual(T.inverse()*T*v,v,"T.inverse()*T*v is not v");
            #RTF.testReport("Running test testTransformInverse for Wrench");
            f = self.randomWrench()
            fZero = iDynTree.Wrench()
            fZero.zero()
            self.checkSpatialVectorEqual(f-f,fZero,"f-f is not zero");
            self.checkSpatialVectorEqual(T.inverse()*T*f,f,"T.inverse()*T*f is not f");

if __name__ == '__main__':
    # initalize the seed to have predictable results
    random.seed(0);
    unittest.main()
